import { IEventParam, Http, Metadata } from '@zeedhi/core';
import { AlertService, DialogService, Form, LoadingService } from '@zeedhi/common';
import { ZdText } from '@zeedhi/vuetify';

type StorageData = {
  [key: string]: string | boolean | number;
};

type StorageMode = 'localstorage' | 'cookie';

type LoginCredentials = {
  email: string,
  password: string,
};

export class LoginController {
  public messageValue?:string;

  form: Array<Form> = [];

  body: any = [];

  /** */
  public gridMounted({ component }: IEventParam<Form>) {
    this.form.push(component);
  }

  public login() {
    const form = Metadata.getInstance<Form>('login-form');

    if (!form.validate()) return;

    this.loginRequest({
      email: form.value.email,
      password: form.value.password,
    });
  }

  public logout() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'Confirmação',
      text: 'Deseja realmente sair do sistema?',
      buttons: [
        { name: 'yes', component: 'ZdButton', label: 'Sim', events: this.doLogout, outline: true },
        { name: 'no', component: 'ZdButton', label: 'Não', events: this.backLogout },
      ],
    });
  }

  private clearCookies() {
    const removeCookie = (cookieName: string) => {
      document.cookie = `${cookieName}=; expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/;`;
    };
    const cookies = ['node', 'nrorg'];
    cookies.forEach(removeCookie);
  }

  private doLogout: any = {
    click: async () => {
      localStorage.clear();
      this.clearCookies();
      window.location.pathname = '/login';
      await this.clearServiceWorkerCache();
      window.location.reload();
    },
  };

  private backLogout: any = {
    click: () => DialogService.hide(),
  };

  public setAppVersion({ component }: IEventParam<ZdText>) {
    const version = process.env.VUE_APP_VERSION || '';
    component.text = version ? `Versão: ${version}` : '';
  }

  private getLoginUrl(): string {
    const {
      VUE_APP_IS_ON_PREMISE: isOnPremise,
      VUE_APP_ENABLE_LOGIN_INTEGRATION: enableLoginIntegration,
      VUE_APP_URL_ASSISTANT_CONTROL: urlAssistantControl,
    } = process.env;

    const useAssistantControl = isOnPremise === 'no' && enableLoginIntegration === 'yes';
    const url = useAssistantControl ? urlAssistantControl : '';
    return `${url}/login`;
  }

  private storeData(storageData: StorageData = {}, type: StorageMode = 'localstorage') {
    Object.keys(storageData).forEach((key: string) => {
      const value = storageData[key];
      if (type === 'localstorage') {
        localStorage.setItem(key, (value || '').toString());
      } else if (type === 'cookie') {
        document.cookie = `${key}=${value}; path=/`;
      }
    });
  }

  private validateLoginResponse(response: any): void {
    if (!response || !response.data) throw new Error('EMPTY_RESPONSE');
    const {
      NRORG,
      TOKEN,
    } = response.data;
    if (!NRORG || !TOKEN) throw new Error('INVALID_AUTH');
  }

  private saveUserInfo(response: any) {
    const {
      NRORG,
      NODE,
      CDOPERADOR,
      NMOPERADOR,
      TOKEN,
      ACCESS_PROFILE: { PERMISSIONS },
    } = response.data;
    this.storeData({ NRORG, NODE, CDOPERADOR, NMOPERADOR, TOKEN, PERMISSIONS }, 'localstorage');
    this.storeData({ nrorg: NRORG, node: NODE }, 'cookie');
  }

  private async auth(credentials: LoginCredentials): Promise<boolean> {
    const url = this.getLoginUrl();
    let response = {};
    const {
      VUE_APP_ASSISTANT_CONTROL_USER: userControl,
      VUE_APP_ASSISTANT_CONTROL_PASSWORD: passwordControl,
    } = process.env;
    try {
      response = await Http.post(url, credentials, {
        headers: {
          Authorization: `Basic ${btoa(`${userControl}:${passwordControl}`)}`,
        },
      });
      this.validateLoginResponse(response);
      this.saveUserInfo(response);
      return true;
    } catch (error) {
      console.error(error);
      AlertService.show({
        name: 'buttons-alert',
        text: 'Falha ao autenticar o usuário/senha. Tente novamente.',
        color: 'error',
        right: true,
        top: true,
        showDismiss: true,
      });
      return false;
    }
  }

  public async clearServiceWorkerCache() {
    const cachedKeys = await caches.keys();
    cachedKeys.forEach((cachedKey) => {
        caches.delete(cachedKey);
    });
  }

  public async loginRequest(data: any) {
    LoadingService.show();

    const isValidAuth = await this.auth(data);
    if (isValidAuth) {
      await this.clearServiceWorkerCache();
      document.location.href = document.location.origin;
    }

    LoadingService.hide();
  }
}
